@@ -46,13 +46,11 @@ namespace {
 class XdsResolver : public Resolver {
  public:
   explicit XdsResolver(ResolverArgs args)
-      : Resolver(std::move(args.work_serializer),
-                 std::move(args.result_handler)),
+      : work_serializer_(std::move(args.work_serializer)),
+        result_handler_(std::move(args.result_handler)),
+        server_name_(absl::StripPrefix(args.uri.path(), "/")),
         args_(grpc_channel_args_copy(args.args)),
         interested_parties_(args.pollset_set) {
-    char* path = args.uri->path;
-    if (path[0] == '/') ++path;
-    server_name_ = path;
     if (GRPC_TRACE_FLAG_ENABLED(grpc_xds_resolver_trace)) {
       gpr_log(GPR_INFO, "[xds_resolver %p] created for server name %s", this,
               server_name_.c_str());
@@ -184,6 +182,8 @@ class XdsResolver : public Resolver {
   void GenerateResult();
   void MaybeRemoveUnusedClusters();
 
+  std::shared_ptr<WorkSerializer> work_serializer_;
+  std::unique_ptr<ResultHandler> result_handler_;
   std::string server_name_;
   const grpc_channel_args* args_;
   grpc_pollset_set* interested_parties_;
@@ -233,7 +233,7 @@ XdsResolver::Notifier::Notifier(RefCountedPtr<XdsResolver> resolver)
 void XdsResolver::Notifier::RunInExecCtx(void* arg, grpc_error* error) {
   Notifier* self = static_cast<Notifier*>(arg);
   GRPC_ERROR_REF(error);
-  self->resolver_->work_serializer()->Run(
+  self->resolver_->work_serializer_->Run(
       [self, error]() { self->RunInWorkSerializer(error); }, DEBUG_LOCATION);
 }
 
@@ -361,28 +361,6 @@ void XdsResolver::XdsConfigSelector::MaybeAddCluster(const std::string& name) {
   }
 }
 
-bool PathMatch(const absl::string_view& path,
-               const XdsApi::Route::Matchers::PathMatcher& path_matcher) {
-  switch (path_matcher.type) {
-    case XdsApi::Route::Matchers::PathMatcher::PathMatcherType::PREFIX:
-      return path_matcher.case_sensitive
-                 ? absl::StartsWith(path, path_matcher.string_matcher)
-                 : absl::StartsWithIgnoreCase(path,
-                                              path_matcher.string_matcher);
-    case XdsApi::Route::Matchers::PathMatcher::PathMatcherType::PATH:
-      return path_matcher.case_sensitive
-                 ? path == path_matcher.string_matcher
-                 : absl::EqualsIgnoreCase(path, path_matcher.string_matcher);
-    case XdsApi::Route::Matchers::PathMatcher::PathMatcherType::REGEX:
-      // Note: Case-sensitive option will already have been set appropriately
-      // in path_matcher.regex_matcher when it was constructed, so no
-      // need to check it here.
-      return RE2::FullMatch(path.data(), *path_matcher.regex_matcher);
-    default:
-      return false;
-  }
-}
-
 absl::optional<absl::string_view> GetMetadataValue(
     const std::string& target_key, grpc_metadata_batch* initial_metadata,
     std::string* concatenated_value) {
@@ -406,61 +384,29 @@ absl::optional<absl::string_view> GetMetadataValue(
   return *concatenated_value;
 }
 
-bool HeaderMatchHelper(
-    const XdsApi::Route::Matchers::HeaderMatcher& header_matcher,
-    grpc_metadata_batch* initial_metadata) {
+bool HeaderMatchHelper(const HeaderMatcher& header_matcher,
+                       grpc_metadata_batch* initial_metadata) {
   std::string concatenated_value;
   absl::optional<absl::string_view> value;
   // Note: If we ever allow binary headers here, we still need to
   // special-case ignore "grpc-tags-bin" and "grpc-trace-bin", since
   // they are not visible to the LB policy in grpc-go.
-  if (absl::EndsWith(header_matcher.name, "-bin") ||
-      header_matcher.name == "grpc-previous-rpc-attempts") {
+  if (absl::EndsWith(header_matcher.name(), "-bin") ||
+      header_matcher.name() == "grpc-previous-rpc-attempts") {
     value = absl::nullopt;
-  } else if (header_matcher.name == "content-type") {
+  } else if (header_matcher.name() == "content-type") {
     value = "application/grpc";
   } else {
-    value = GetMetadataValue(header_matcher.name, initial_metadata,
+    value = GetMetadataValue(header_matcher.name(), initial_metadata,
                              &concatenated_value);
   }
-  if (!value.has_value()) {
-    if (header_matcher.type ==
-        XdsApi::Route::Matchers::HeaderMatcher::HeaderMatcherType::PRESENT) {
-      return !header_matcher.present_match;
-    } else {
-      // For all other header matcher types, we need the header value to
-      // exist to consider matches.
-      return false;
-    }
-  }
-  switch (header_matcher.type) {
-    case XdsApi::Route::Matchers::HeaderMatcher::HeaderMatcherType::EXACT:
-      return value.value() == header_matcher.string_matcher;
-    case XdsApi::Route::Matchers::HeaderMatcher::HeaderMatcherType::REGEX:
-      return RE2::FullMatch(value.value().data(), *header_matcher.regex_match);
-    case XdsApi::Route::Matchers::HeaderMatcher::HeaderMatcherType::RANGE:
-      int64_t int_value;
-      if (!absl::SimpleAtoi(value.value(), &int_value)) {
-        return false;
-      }
-      return int_value >= header_matcher.range_start &&
-             int_value < header_matcher.range_end;
-    case XdsApi::Route::Matchers::HeaderMatcher::HeaderMatcherType::PREFIX:
-      return absl::StartsWith(value.value(), header_matcher.string_matcher);
-    case XdsApi::Route::Matchers::HeaderMatcher::HeaderMatcherType::SUFFIX:
-      return absl::EndsWith(value.value(), header_matcher.string_matcher);
-    default:
-      return false;
-  }
+  return header_matcher.Match(value);
 }
 
-bool HeadersMatch(
-    const std::vector<XdsApi::Route::Matchers::HeaderMatcher>& header_matchers,
-    grpc_metadata_batch* initial_metadata) {
+bool HeadersMatch(const std::vector<HeaderMatcher>& header_matchers,
+                  grpc_metadata_batch* initial_metadata) {
   for (const auto& header_matcher : header_matchers) {
-    bool match = HeaderMatchHelper(header_matcher, initial_metadata);
-    if (header_matcher.invert_match) match = !match;
-    if (!match) return false;
+    if (!HeaderMatchHelper(header_matcher, initial_metadata)) return false;
   }
   return true;
 }
@@ -475,8 +421,8 @@ ConfigSelector::CallConfig XdsResolver::XdsConfigSelector::GetCallConfig(
     GetCallConfigArgs args) {
   for (const auto& entry : route_table_) {
     // Path matching.
-    if (!PathMatch(StringViewFromSlice(*args.path),
-                   entry.route.matchers.path_matcher)) {
+    if (!entry.route.matchers.path_matcher.Match(
+            StringViewFromSlice(*args.path))) {
       continue;
     }
     // Header Matching.
@@ -545,7 +491,7 @@ ConfigSelector::CallConfig XdsResolver::XdsConfigSelector::GetCallConfig(
           GRPC_CLOSURE_CREATE(
               [](void* arg, grpc_error* /*error*/) {
                 auto* resolver = static_cast<XdsResolver*>(arg);
-                resolver->work_serializer()->Run(
+                resolver->work_serializer_->Run(
                     [resolver]() {
                       resolver->MaybeRemoveUnusedClusters();
                       resolver->Unref();
@@ -572,7 +518,7 @@ void XdsResolver::StartLocked() {
             "Failed to create xds client -- channel will remain in "
             "TRANSIENT_FAILURE: %s",
             grpc_error_string(error));
-    result_handler()->ReturnError(error);
+    result_handler_->ReturnError(error);
     return;
   }
   grpc_pollset_set_add_pollset_set(xds_client_->interested_parties(),
@@ -664,7 +610,7 @@ void XdsResolver::OnError(grpc_error* error) {
   Result result;
   result.args = grpc_channel_args_copy(args_);
   result.service_config_error = error;
-  result_handler()->ReturnResult(std::move(result));
+  result_handler_->ReturnResult(std::move(result));
 }
 
 void XdsResolver::OnResourceDoesNotExist() {
@@ -678,7 +624,7 @@ void XdsResolver::OnResourceDoesNotExist() {
       ServiceConfig::Create(args_, "{}", &result.service_config_error);
   GPR_ASSERT(result.service_config != nullptr);
   result.args = grpc_channel_args_copy(args_);
-  result_handler()->ReturnResult(std::move(result));
+  result_handler_->ReturnResult(std::move(result));
 }
 
 grpc_error* XdsResolver::CreateServiceConfig(
@@ -736,7 +682,7 @@ void XdsResolver::GenerateResult() {
   }
   grpc_arg new_arg = config_selector->MakeChannelArg();
   result.args = grpc_channel_args_copy_and_add(args_, &new_arg, 1);
-  result_handler()->ReturnResult(std::move(result));
+  result_handler_->ReturnResult(std::move(result));
 }
 
 void XdsResolver::MaybeRemoveUnusedClusters() {
@@ -762,8 +708,8 @@ void XdsResolver::MaybeRemoveUnusedClusters() {
 
 class XdsResolverFactory : public ResolverFactory {
  public:
-  bool IsValidUri(const grpc_uri* uri) const override {
-    if (GPR_UNLIKELY(0 != strcmp(uri->authority, ""))) {
+  bool IsValidUri(const URI& uri) const override {
+    if (GPR_UNLIKELY(!uri.authority().empty())) {
       gpr_log(GPR_ERROR, "URI authority not supported");
       return false;
     }
